/*
 * This file: list_text_exp.cpp
 *
 * DO NOT MODIFY THIS FILE.
 */
#include <string>
#include "Name.h"
#include "Movie.h"
#include "CLList.cpp"
#include "queue.cpp"
#include "BinarySearchTree.cpp"

using namespace std;
using namespace uprm_icom4035;


// Explicit binding of templates

// String bindings
template class cl_list_node<string>;

template class list<string>;

template class Iterator<string>;

// List of strings
template class CLList<string>;

// Iterator of strings
template class CLLIterator<string>;

// Name bindings
template class cl_list_node<Name>;

template class list<Name>;

template class Iterator<Name>;

// List of Names
template class CLList<Name>;

// Iterator of Names
template class CLLIterator<Name>;

// Movie bindings
template class cl_list_node<Movie>;

template class list<Movie>;

template class Iterator<Movie>;

// List of Movies
template class CLList<Movie>;

// Iterator of Movies
template class CLLIterator<Movie>;

// Binary Search Tree of movies
template class BinarySearchTree<Movie,string>;

// In-Order Tree iterator for Binary Search Tree of movies
template class BST_inorder_iterator<Movie,string>;

// Pre-Order Tree iterator for Binary Search Tree of movies
template class BST_preorder_iterator<Movie,string>;
