/*
 * This file name: DataManager.cpp
 * This file contains the implementation of the methods for the
 * data manager class, used to manipulate the data in the 
 * movie database application.
 */

#include "DataManager.h"

namespace uprm_icom4035 {
  ////////////////////////// DO NOT MODIFY THIS FUNCTION !!! /////////////////
  void DataManager::print_tree_height() const {
    cout << "BST Height: " << movie_bst.height() << endl;
  }
  ///////////////////////////////////////////////////////////////////////////
  
  /***************** START ADDING YOUR CODE HERE **************************/

}
