#include <string>
#include "Name.h"
#include "queue.cpp"
#include "BinarySearchTree.cpp"

using namespace std;
using namespace uprm_icom4035;

// Explicit template class declaration
// Here we include all files with template declaration so
// they can be checked and bind by the C++ compiler

// List of node pointers
//template class cdlist_node<BSTNode<Name>* >;

// Queue of Node pointers
//template class queue<BSTNode<Name>* >;

// Binary Search Tree of names
template class BinarySearchTree<Name,string>;

// Inorder Tree iterator for Binary Search Tree of names
template class BST_inorder_iterator<Name,string>;

// Preorder Tree iterator for Binary Search Tree of names
template class BST_preorder_iterator<Name,string>;





