/*
 * This file: queue_test_exp.cpp
 * 
 * DO NOT MODIFY THIS FILE!!!
 */
#include <string>
#include "queue.cpp"
#include "Name.h"
using namespace std;
using namespace uprm_icom4035;

// Explicit binding of templates

// Node of Name

template class cl_list_node<Name>;

// Queue of Name
template class queue<Name>;
 
