#include <iostream>
#include <cstdlib>
#include <cassert>
#include "CLList.h"

namespace uprm_icom4035 {
  
  //////////////// DO NOT MODIFY THIS FUNCTION //////////////////////////////
  template <typename Item>
  void CLList<Item>::print_list(ostream& out) const{
    cl_list_node<Item> *temp = NULL;
    
    // print each data value on a separate line
    for (temp = header->next; temp != header; temp = temp->next){
      out << temp->data << endl;
    }
  } 
  
  //////////////////////////////////////////////////////////////////////////

  /************* START ADDING FOR CODE FROM HERE **************************/
 
}
