/*
 * Department of Electrical and
 * Computer Engineering
 * University of Puerto Rico
 * Mayaguez Campus
 *
 * ICOM 4035 - Data Structures
 * Spring 2003
 * Dr. Manuel Rodriguez-Martinez
 *
 * This file name: list_test.h
 *
 * This file contains the interface with the methods
 * for the program used to interact with the SortedSLL.
 *
 */

#ifndef _LIST_TEST_H
#define _LIST_TEST_H

#include "list.h"
#include "Iterator.h"

using namespace std;
using namespace uprm_icom4035;

#define LIST_INSERT  1 
#define LIST_FIND    2
#define LIST_FIRST   3
#define LIST_ERASE   4
#define LIST_SIZE    5
#define LIST_ISEMPTY 6
#define LIST_MEMPTY  7
#define LIST_PRINT   8
#define LIST_DUP     9
#define LIST_EXIT    10


list *make_list();
void print_menu();
int get_option();

void list_insert(list* L);
void list_find(list* L);
void list_first(list* L);
void list_erase(list* L);
void list_size(list* L);
void list_isempty(list* L);
void list_mempty(list* L);
void list_print(list* L);
void list_dup(list* L);

#endif
